<?php

namespace FiloBlu\Rma\Plugin\Rma\Api\RmaRepositoryInterface;


use Magento\Framework\App\ResourceConnection;
use Magento\Sales\Api\OrderRepositoryInterface;
use Psr\Log\LoggerInterface;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Magento\Rma\Model\ResourceModel\Item\CollectionFactory as ItemCollectionFactory;
use FiloBlu\Rma\Helper\RmaHelper;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use FiloBlu\Rma\Api\EasyReturnRepositoryInterface;
use Magento\Sales\Api\Data\OrderAddressInterfaceFactory;

class AfterGet
{
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var \Magento\Rma\Api\Data\ItemExtensionFactory
     */
    private $extensionInterfaceFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    protected $rmaExtensionAttributesRepository;

    /**
     * @var ItemCollectionFactory
     */
    protected $itemCollectionFactory;
    /**
     * @var RmaHelper
     */
    private $rmaHelper;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;
    /**
     * @var CustomerRepositoryInterface
     */
    private $customerRepository;

    /**
     * @var \Magento\Rma\Api\Data\RmaExtensionInterfaceFactory
     */
    private $rmaExtensionInterfaceFactory;
    /**
     * @var EasyReturnRepositoryInterface
     */
    private $easyReturnRepository;
    /**
     * @var OrderAddressInterfaceFactory
     */
    private $addressInterfaceFactory;


    /**
     * AfterGet constructor.
     * @param OrderRepositoryInterface $orderRepository
     * @param \Magento\Rma\Api\Data\ItemExtensionFactory $extensionInterfaceFactory
     * @param ResourceConnection $resourceConnection
     * @param LoggerInterface $logger
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param ItemCollectionFactory $itemCollectionFactory
     * @param RmaHelper $rmaHelper
     * @param ProductRepositoryInterface $productRepository
     * @param CustomerRepositoryInterface $customerRepository
     * @param \Magento\Rma\Api\Data\RmaExtensionInterfaceFactory $rmaExtensionInterfaceFactory
     * @param EasyReturnRepositoryInterface $easyReturnRepository
     * @param OrderAddressInterfaceFactory $addressInterfaceFactory
     */
    public function __construct(
        OrderRepositoryInterface $orderRepository,
        \Magento\Rma\Api\Data\ItemExtensionFactory $extensionInterfaceFactory,
        ResourceConnection $resourceConnection,
        LoggerInterface $logger,
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        ItemCollectionFactory $itemCollectionFactory,
        RmaHelper $rmaHelper,
        ProductRepositoryInterface $productRepository,
        CustomerRepositoryInterface $customerRepository,
        \Magento\Rma\Api\Data\RmaExtensionInterfaceFactory $rmaExtensionInterfaceFactory,
        EasyReturnRepositoryInterface $easyReturnRepository,
        OrderAddressInterfaceFactory $addressInterfaceFactory)

    {
        $this->orderRepository = $orderRepository;
        $this->extensionInterfaceFactory = $extensionInterfaceFactory;
        $this->resourceConnection = $resourceConnection;
        $this->logger = $logger;
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->itemCollectionFactory = $itemCollectionFactory;
        $this->rmaHelper = $rmaHelper;
        $this->productRepository = $productRepository;
        $this->customerRepository = $customerRepository;
        $this->rmaExtensionInterfaceFactory = $rmaExtensionInterfaceFactory;
        $this->easyReturnRepository = $easyReturnRepository;
        $this->addressInterfaceFactory = $addressInterfaceFactory;
    }

    /**
     * @param \Magento\Rma\Api\RmaRepositoryInterface $subject
     * @param \Magento\Rma\Api\Data\RmaInterface $result
     * @return \Magento\Rma\Api\Data\RmaInterface
     */
    public function afterGet(\Magento\Rma\Api\RmaRepositoryInterface $subject, \Magento\Rma\Api\Data\RmaInterface $result)
    {
        try{
            // TODO: Find better solution, Duck typing for now ..
            if(method_exists($result, 'setData')) {
                $result->setData('is_easy_return', false);
            }
            $needEnrichment = false;
            if($result->getItems()){
                //check if condition , reason and resolution are setted
                foreach ($result->getItems() as $item){
                    if(!$item->getCondition() || !$item->getResolution() || !$item->getReason()){
                        $needEnrichment = true;
                        break;
                    }
                }
            }
            //backward compatibility
            if($result->getOrderId() && !$result->getItems() || $needEnrichment) {
                $orderItems = $this->orderRepository->get($result->getOrderId())->getItems();
                $tmpItems = [];
                foreach ($orderItems as $orderItem){
                    $orderItemId = $orderItem->getItemId();
                    $tmpItems = $tmpItems + $this->itemCollectionFactory->create()->addFieldToSelect('*')
                            ->addFieldToFilter('order_item_id',$orderItemId)
                            ->addFieldToFilter('rma_entity_id',$result->getEntityId())
                            ->getItems();
                }
                $rmaItems = $tmpItems;
                $result->setItems($rmaItems);
            }else{
                $rmaItems = $result->getItems();
            }

            foreach ($rmaItems as $rmaItem) {

                $attributes = $rmaItem->getExtensionAttributes();
                if ($attributes === null) {
                    /** @var \Magento\Rma\Api\Data\ItemExtensionFactory $attributes */
                    $attributes = $this->extensionInterfaceFactory->create();
                }

                //sku
                $productSku = $rmaItem->getProductSku();
                $attributes->setSku($productSku);

                $product = $this->productRepository->get($rmaItem->getProductSku());

                //barcode
                $productBarcode = $product->getBarcode();
                $attributes->setBarcode($productBarcode);


                //name
                $productName =$rmaItem->getProductName();
                $attributes->setName($productName);

                //weight
                $weight = $product->getWeight();
                $attributes->setWeight($weight);

                if($changeSizeOrderId = $this->rmaExtensionAttributesRepository->getChangeSizeOrderId($rmaItem->getEntityId())){
                    //change_size_order_id
                    $attributes->setChangeSizeOrderId($changeSizeOrderId);
                    $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);
                    //change_size_increment_id
                    $attributes->setChangeSizeOrderIncrementId($changeSizeOrder->getIncrementId());
                    foreach ($changeSizeOrder->getItems() as $item){
                        //change_size_sku
                        $attributes->setChangeSizeSku($item->getSku());
                    }
                }

                //standard_reason
                if($rmaItem->getReason() !== null){
                    $attributes->setStandardReason($this->getStandardReasonByReasonId($rmaItem->getReason()));
                }

                //save extension attributes
                $rmaItem->setExtensionAttributes($attributes);

            }

            $rmaAttributes = $result->getExtensionAttributes();
            if ($rmaAttributes === null) {
                /** @var \Magento\Rma\Api\Data\RmaExtensionInterfaceFactory $rmaAttributes */
                $rmaAttributes = $this->rmaExtensionInterfaceFactory->create();
            }

            $order = $this->orderRepository->get($result->getOrderId());

            //ext_order_id
            $rmaAttributes->setExtOrderId($order->getExtOrderId());

            //customer_firstname
            $rmaAttributes->setCustomerFirstname($order->getCustomerFirstname());

            //customer_lastname
            $rmaAttributes->setCustomerLastname($order->getCustomerLastname());

            //theoric shipping amount
            $shippingAmount = $this->getTheoricShippingAmount($result->getEntityId());
            if($shippingAmount!=""){
                $rmaAttributes->setTheoricShippingAmount($shippingAmount);
            }


            //easy_return
            $easyReturn = $this->getEasyReturn($result->getEntityId());
            $rmaAttributes->setEasyReturn($easyReturn);

            //pickup Address
            if($easyReturn){
                $pickUpAddress = $this->getPickupAddress($result);
                $rmaAttributes->setPickupAddress($pickUpAddress);
                // TODO: Find better solution, Duck typing for now ..
                if(method_exists($result, 'setData')) {
                    $result->setData('is_easy_return', true);
                }
            }
            //save
            $result->setExtensionAttributes($rmaAttributes);

            return $result;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return $result;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return $result;
        }

    }

    /**
     * @param int $reasonId
     * @return string
     */
    public function getStandardReasonByReasonId(int $reasonId){

        $rmaReasonId = $reasonId;

        if($this->rmaHelper->checkIfItIsMapped($rmaReasonId)){
            $standardReason = $this->rmaHelper->getMappedReasonByReasonId($rmaReasonId,'brand');
            if($standardReason) {
                return $standardReason[0];
            }else{
                return "";
            }
        }

        return "";
    }

    /**
     * @param int $rmaId
     * @return string
     */
    protected function getTheoricShippingAmount(int $rmaId){
        if($rmaId){
            $easyReturn = $this->easyReturnRepository->getByRmaId($rmaId);
            return $easyReturn->getTheoricShippingAmount();
        }
        return "";
    }

    /**
     * @param int $rmaId
     * @return bool
     */
    protected function getEasyReturn(int $rmaId){
        if($rmaId){
            $easyReturn = $this->easyReturnRepository->getByRmaId($rmaId);
            return $easyReturn->getId() ? true : false;
        }
        return false;
    }

    /**
     * @param \Magento\Rma\Api\Data\RmaInterface $rma
     * @return \Magento\Sales\Api\Data\OrderAddressInterface
     */
    protected function getPickupAddress(\Magento\Rma\Api\Data\RmaInterface $rma){

        if($this->getEasyReturn($rma->getEntityId())){
            $easyReturn = $this->easyReturnRepository->getByRmaId($rma->getEntityId());
            return $easyReturn->getPickUpAddress();
        }else{
            //$order = $this->orderRepository->get($rma->getOrderId());
            //return $shippingAddress = $order->getShippingAddress();
            return $this->addressInterfaceFactory->create();
        }
    }



}
